
//Key for the preference file numbers:
//  10 = Standard Mode (decimal)
//  16 = Hexidecimary
//  2 = Binary
//  9 = ASCII Decimal
//  1 = ASCII Binary
//  3 = Equation Mode

//Uncomment to enable debugger mode
//#define GCDEBUGGER

#import "GCAppController.h"
#import "GCLoginItemController.h"

//tgmath.h for CGFloat rounding in the hour section of Equation Mode
#include <tgmath.h>

@implementation GCAppController

NSInteger preferenceNumber;
BOOL loginBool;

-(void) awakeFromNib {
    //Create the NSStatusBar and set variable length
    statusItem = [[NSStatusBar systemStatusBar] statusItemWithLength: NSVariableStatusItemLength];
    
    //Turn off Dock icon
    [NSApp setActivationPolicy:NSApplicationActivationPolicyProhibited];
    
    [self tick:nil];
    
    [GCLoginItemController addAppAsLoginItemWithAllUsers: NO];
    
    //Sets the initial menu title for the NSStatusItem
    [statusItem setTitle: @"[G-Clock checking for preference file, please wait]"];
    
    //disables all menu items
    [_standardMenuItem setEnabled: NO];
    [_hexMenuItem setEnabled: NO];
    [_binaryMenuItem setEnabled: NO];
    [_decASCII_MenuItem setEnabled: NO];
    [_binASCII_MenuItem setEnabled: NO];
    [_equationMenuItem setEnabled: NO];
    
    //Checks to see if preference file is existing.
    //Defines file paths.
    NSString *dirName = [@"~/Library/Application Support/GClockData" stringByExpandingTildeInPath];
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    NSFileManager *fm = [NSFileManager defaultManager];
    
    //Makes sure directory exists and has correct permissions, creates it if not created already.
    while (1) {
        if ([fm fileExistsAtPath: dirName] == YES) {
            if ([fm isWritableFileAtPath: dirName] == YES) {
                if ([fm isReadableFileAtPath: dirName] == YES) {
                    break;
                } else {
#                   ifdef GCDEBUGGER
                    NSLog(@"Permissions error.");
#                   endif
                }
            } else {
#               ifdef GCDEBUGGER
                NSLog(@"Permissions error.");
#               endif
            }
        } else {
            //create directory
            if ([fm createDirectoryAtPath: dirName withIntermediateDirectories: YES attributes: nil error: NULL] != YES){
#               ifdef GCDEBUGGER
                NSLog(@"Save File Creation Error. (%@)", dirName);
#               endif
            }
        }
    }
    //Makes sure file exists and has correct permissions, creates it if not created already.
    while (1) {
        if ([fm fileExistsAtPath: fileName] ==YES) {
            if ([fm isWritableFileAtPath: fileName] == YES) {
                if ([fm isReadableFileAtPath: fileName] == YES) {
                    break;
                } else {
#                   ifdef GCDEBUGGER
                    NSLog(@"Permissions error.");
#                   endif
                }
            } else {
#               ifdef GCDEBUGGER
                NSLog(@"Permissions error.");
#               endif
            }
        } else {
            NSString *number = @"3";
            BOOL loginBool = YES;
            NSString *string = [NSString stringWithFormat:@"%@@%hhd", number, loginBool];
            NSData *stringData = [string dataUsingEncoding:NSUTF8StringEncoding];
            //create file
            if ([fm createFileAtPath: fileName contents: stringData attributes: nil] != YES)
                NSLog (@"Save File Creation Error. (%@)", dirName);
        }
    }
    
    //Gets preferences
    NSString *fileContentsString = [NSString stringWithContentsOfFile: fileName encoding: NSUTF8StringEncoding error:NULL];
    NSArray *fileContents = [fileContentsString componentsSeparatedByString:@"@"];
    
    //Sets preferencs
    preferenceNumber = [fileContents[0] intValue];
    loginBool = [fileContents[1] boolValue];
    
    //Sets loginItem's state
    if (loginBool == YES) {
        [_loginItem setState: NSOnState];
    } else {
        [_loginItem setState: NSOffState];
    }
    
    int prefNumber = (int)preferenceNumber;
    
#   ifdef GCDEBUGGER
    NSLog (@"GCDEBUGGER MESSAGE: preferenceNumber: %ld", preferenceNumber);
    NSLog (@"GCDEBUGGER MESSAGE: prefNumber: %i", prefNumber);
#   endif
    
    //Tells NSStatusItem what menu to load
    [statusItem setMenu:statusMenu];
    
    //Sets tooltip for item
    [statusItem setToolTip:@"Menu Item"];
    
    //Sets initial disablement/enablement
    if (prefNumber == 10) {
        [_standardMenuItem setState: NSOnState];
        [_hexMenuItem setState: NSOffState];
        [_binaryMenuItem setState: NSOffState];
        [_decASCII_MenuItem setState: NSOffState];
        [_binASCII_MenuItem setState: NSOffState];
        [_equationMenuItem setState: NSOffState];
        [_standardMenuItem setEnabled: NO];
        [_hexMenuItem setEnabled: YES];
        [_binaryMenuItem setEnabled: YES];
        [_decASCII_MenuItem setEnabled: YES];
        [_binASCII_MenuItem setEnabled: YES];
        [_equationMenuItem setEnabled: YES];
    }else if (prefNumber == 16) {
        [_standardMenuItem setState: NSOffState];
        [_hexMenuItem setState: NSOnState];
        [_binaryMenuItem setState: NSOffState];
        [_decASCII_MenuItem setState: NSOffState];
        [_binASCII_MenuItem setState: NSOffState];
        [_equationMenuItem setState: NSOffState];
        [_standardMenuItem setEnabled: YES];
        [_hexMenuItem setEnabled: NO];
        [_binaryMenuItem setEnabled: YES];
        [_decASCII_MenuItem setEnabled: YES];
        [_binASCII_MenuItem setEnabled: YES];
        [_equationMenuItem setEnabled: YES];
    } else if (prefNumber == 2) {
        [_standardMenuItem setState: NSOffState];
        [_hexMenuItem setState: NSOffState];
        [_binaryMenuItem setState: NSOnState];
        [_decASCII_MenuItem setState: NSOffState];
        [_binASCII_MenuItem setState: NSOffState];
        [_equationMenuItem setState: NSOffState];
        [_standardMenuItem setEnabled: YES];
        [_hexMenuItem setEnabled: YES];
        [_binaryMenuItem setEnabled: NO];
        [_decASCII_MenuItem setEnabled: YES];
        [_binASCII_MenuItem setEnabled: YES];
        [_equationMenuItem setEnabled: YES];
    } else if (prefNumber == 9) {
        [_standardMenuItem setState: NSOffState];
        [_hexMenuItem setState: NSOffState];
        [_binaryMenuItem setState: NSOffState];
        [_decASCII_MenuItem setState: NSOnState];
        [_binASCII_MenuItem setState: NSOffState];
        [_equationMenuItem setState: NSOffState];
        [_standardMenuItem setEnabled: YES];
        [_hexMenuItem setEnabled: YES];
        [_binaryMenuItem setEnabled: YES];
        [_decASCII_MenuItem setEnabled: NO];
        [_binASCII_MenuItem setEnabled: YES];
        [_equationMenuItem setEnabled: YES];
    } else if (prefNumber == 1) {
        [_standardMenuItem setState: NSOffState];
        [_hexMenuItem setState: NSOffState];
        [_binaryMenuItem setState: NSOffState];
        [_decASCII_MenuItem setState: NSOffState];
        [_binASCII_MenuItem setState: NSOnState];
        [_equationMenuItem setState: NSOffState];
        [_standardMenuItem setEnabled: YES];
        [_hexMenuItem setEnabled: YES];
        [_binaryMenuItem setEnabled: YES];
        [_decASCII_MenuItem setEnabled: YES];
        [_binASCII_MenuItem setEnabled: NO];
        [_equationMenuItem setEnabled: YES];
    } else if (prefNumber == 3) {
        [_standardMenuItem setState: NSOffState];
        [_hexMenuItem setState: NSOffState];
        [_binaryMenuItem setState: NSOffState];
        [_decASCII_MenuItem setState: NSOffState];
        [_binASCII_MenuItem setState: NSOffState];
        [_equationMenuItem setState: NSOnState];
        [_standardMenuItem setEnabled: YES];
        [_hexMenuItem setEnabled: YES];
        [_binaryMenuItem setEnabled: YES];
        [_decASCII_MenuItem setEnabled: YES];
        [_binASCII_MenuItem setEnabled: YES];
        [_equationMenuItem setEnabled: NO];
    } else {
        [_standardMenuItem setState: NSOffState];
        [_hexMenuItem setState: NSOffState];
        [_binaryMenuItem setState: NSOffState];
        [_decASCII_MenuItem setState: NSOffState];
        [_binASCII_MenuItem setState: NSOffState];
        [_equationMenuItem setState: NSOffState];
        [_standardMenuItem setEnabled: YES];
        [_hexMenuItem setEnabled: YES];
        [_binaryMenuItem setEnabled: YES];
        [_decASCII_MenuItem setEnabled: YES];
        [_binASCII_MenuItem setEnabled: YES];
        [_equationMenuItem setEnabled: YES];
    }
}

-(void)tick:(id)sender {
    
    //Gets current date and puts it as a string
    NSDate *date = [NSDate date];
    
    NSDateFormatter *dateFormatterHour = [[NSDateFormatter alloc] init];
    [dateFormatterHour setDateFormat:@"hh"];
    NSString *stringFromDateHour = [dateFormatterHour stringFromDate: date];
    unsigned int hour = [stringFromDateHour intValue];
    
    int prefNumber = (int)preferenceNumber;
    
    
    NSString *stringHour;
    if (prefNumber == 16)
        stringHour = [NSString stringWithFormat:@"%X", hour];
    else if (prefNumber == 10)
        stringHour = [NSString stringWithFormat:@"%u", hour];
    else if (prefNumber == 2){
        NSMutableString *str = [NSMutableString stringWithFormat:@""];
        for(NSInteger numberCopy = hour; numberCopy > 0; numberCopy >>=1) {
            //Prepend "0" or "1", depending on the bit
            [str insertString:((numberCopy & 1) ? @"1" : @"0") atIndex:0];
        }
        stringHour = [NSString stringWithFormat:@"%@", str];
    } else if (prefNumber == 9) {
        //Convert to dec ASCII
        NSMutableString * str = [NSMutableString stringWithFormat:@""];
        NSString *hourString = [NSString stringWithFormat:@"%i", hour];
        for (int i = 0; i < [hourString length]; ++i) {
            char c = [hourString characterAtIndex:i];
            int ASCII_Code = (int)c;
            NSString *ASCII_String = [NSString stringWithFormat: @"%i", ASCII_Code];
            [str appendString:ASCII_String];
            if (i + 1 != [hourString length])
            [str appendString:@" "];
        }
#       ifdef GCDEBUGGER
        NSLog (@"str hour: %@", str);
#       endif 
        stringHour = str;
    } else if (prefNumber == 1) {
        //Convert to dec ASCII
        NSMutableString * str = [NSMutableString stringWithFormat:@""];
        NSString *hourString = [NSString stringWithFormat:@"%i", hour];
        for (int i = 0; i < [hourString length]; ++i) {
            char c = [hourString characterAtIndex:i];
            int ASCII_Code = (int)c;
            NSString *ASCII_String = [NSString stringWithFormat: @"%i", ASCII_Code];
            [str appendString:ASCII_String];
            if (i + 1 != [hourString length])
                [str appendString:@" "];
        }
#       ifdef GCDEBUGGER
        NSLog (@"str hour: %@", str);
#       endif
        
        //Convert to binary
        NSMutableString *str2 = [NSMutableString stringWithFormat:@""];
        NSArray *array = [str componentsSeparatedByString: @" "];
        NSUInteger numObjects = [array count];
        //Converts each object of array into binary separately, so spaces remain intact.
        for (int i = 0; i < numObjects; ++i) {
            NSString *string = array[i];
            NSMutableString *mutStr = [NSMutableString stringWithFormat:@""];
            for (NSInteger numberCopy = [string intValue]; numberCopy > 0; numberCopy >>=1) {
                //Prepend "0" or "1", depending on the bit
                [mutStr insertString:((numberCopy & 1) ? @"1" : @"0") atIndex:0];
            }
            [str2 appendString: mutStr];
            if (numObjects != i + 1) {
                //Add space if this is not last run through
                [str2 appendString: @" "];
            }
        }
        stringHour = str2;
    } else if (prefNumber == 3) {
        //Hour Equation Mode is the hardest difficulty.
        
        if (_hourNumber != hour) {
            
            NSInteger random, random3;
            NSMutableString *hourString = [NSMutableString stringWithFormat:@" "];
            
            //Declares how many operations will be in hour
            random3 = 3 + (NSInteger)(arc4random_uniform(4));
            
#           ifdef GCDEBUGGER
            NSLog (@"GCDEBUGGER MESSAGE: random3: %ld", (long)random3);
#           endif
            
            for (int i = 0; i <= random3; ++i) {
                //Decides which section to go through (addition & subtraction (50%), multiplication (25%), and division (25%))
                random = 1 + (NSInteger)(arc4random_uniform(4));
                
                NSString *appendString;

                if (i == 0) _answer = hour;

                //Tells what the answer of the equation should be on any runs through the program after the first one.
                NSInteger answer = _answer;
                
                if ((random == 1) || (random == 2)) {
                    //Addition and Subtraction
                    
                    NSInteger random2 = 49 - (NSInteger)(arc4random_uniform(100));
                    NSNumber *numb = [NSNumber numberWithInteger: answer - random2];
                    //Turns numb into string, if positive "+ #" if negative "- #", and if zero a random chance of either.
                    NSString *numbString;
                    if ([numb intValue] > 0) {
                        numbString = [NSString stringWithFormat:@"+ %@", numb];
                    } else if ([numb intValue] < 0) {
                        NSMutableString *stringFromNumber = [NSMutableString stringWithFormat:@"%@", numb];
                        //Adds space right after the negative operator in the string -# so it will later read # - # instead of -#
                        [stringFromNumber insertString: @" " atIndex:1];
                        numbString = [NSString stringWithFormat:@"%@", stringFromNumber];
                    } else if ([numb intValue] == 0) {
                        NSUInteger randomTemp = 1 + (NSInteger)(arc4random_uniform(2));
                        if (randomTemp == 1)
                            numbString = [NSString stringWithFormat:@"+ %@", numb];
                        else if (randomTemp == 2)
                            numbString = [NSString stringWithFormat:@"- %@", numb];
                    }
                    
                    if (random3 == i) {
                        //Last run through of for block, replaces the placeholder ("@") with appendString. appendString doesn't get a placeholder ("@"), as another string will not be inserted
                        appendString = [NSString stringWithFormat: @"%li %@", random2, numbString];
                        
                        //Finds range of placeholder.
                        NSRange range = [hourString rangeOfString:@"@"];
                        //Replaces @ with appendString.
                        [hourString replaceCharactersInRange: range withString: appendString];
                    } else if (i == 0) {
                        //First run through of for block, defines NSMutableString as what would be appendString, as there is nothing yet to be replaced.
                        hourString = [NSMutableString stringWithFormat: @"@ %@", numbString];
                    } else {
                        //In between first and last run of for block, replaces the placeholder ("@") with appendString, and has appendString have a placeholder("@), for the next string to be inserted in.
                        appendString = [NSString stringWithFormat: @"@ %@", numbString];
                        
                        //Finds range of placeholder.
                        NSRange range = [hourString rangeOfString:@"@"];
                        //Replaces @ with appendString.
                        [hourString replaceCharactersInRange: range withString: appendString];
                    }
                    
#                   ifdef GCDEBUGGER
                    NSLog (@"GCDEBUGGER MESSAGE: +- i: %i answer: %ld _answer: %ld random2: %ld numb: %@ numbString: %@ hourString: %@", i, (long)answer, (long)_answer, (long)random2, numb, numbString, hourString);
#                   endif
                    
                    _answer = random2;
                    
                    _hourNumber = hour;
                    stringHour = hourString;
                } else if (random == 3) {
                    //Multiplication
                    
                    NSInteger random2 = 1 + (NSInteger)(arc4random_uniform(20));

                    float random2F= random2;
                    float answerF = answer;
                    CGFloat floatForNumb = (answerF / random2F);
                    //Rounds to nearest hundredth by multiplying by 100, rounding, and then dividing by 100
                    CGFloat floatForNumbRounded = roundf(floatForNumb * 100) / 100;
                    //You have to convert it into a NSNumber so it auto-truncates insignificant digits.
                    NSNumber *numb = [NSNumber numberWithFloat:floatForNumbRounded];
                    
                    if (random3 == i) {
                        //Last run through of for block, replaces the placeholder ("@") with appendString. appendString doesn't get a placeholder ("@"), as another string will not be inserted
                        appendString = [NSString stringWithFormat: @"%ld * %@", (long)random2, numb];
                        
                        //Finds range of placeholder.
                        NSRange range = [hourString rangeOfString:@"@"];
                        //Replaces @ with appendString.
                        [hourString replaceCharactersInRange: range withString: appendString];
                    } else if (i == 0) {
                        //First run through of for block, defines NSMutableString as what would be appendString, as there is nothing yet to be replaced.
                        hourString = [NSMutableString stringWithFormat: @"(@) * %@", numb];
                    } else {
                        //In between first and last run of for block, replaces the placeholder ("@") with appendString, and has appendString have a placeholder("@), for the next string to be inserted in.
                        appendString = [NSString stringWithFormat: @"(@) * %@", numb];
                        
                        //Finds range of placeholder.
                        NSRange range = [hourString rangeOfString:@"@"];
                        //Replaces @ with appendString.
                        [hourString replaceCharactersInRange: range withString: appendString];
                    }
                    
#                   ifdef GCDEBUGGER
                    NSLog (@"GCDEBUGGER MESSAGE: * i: %i answer: %ld _answer: %ld random2: %ld numb: %@ hourString: %@", i, (long)answer, (long)_answer, (long)random2, numb, hourString);
#                   endif
                    
                    _answer = random2;
                    
                    _hourNumber = hour;
                    stringHour = hourString;
                } else if (random == 4) {
                    //Division
                    
                    NSInteger random2 = 1 + (NSInteger)(arc4random_uniform(20));
                    
                    
                    float random2F= random2;
                    float answerF = answer;
                    CGFloat floatForNumb = (random2F / answerF);
                    //Rounds to nearest hundredth by multiplying by 100, rounding, and then dividing by 100
                    CGFloat floatForNumbRounded = roundf(floatForNumb * 100) / 100;
                    //You have to convert it into a NSNumber so it auto-truncates insignificant digits.
                    NSNumber *numb = [NSNumber numberWithFloat:floatForNumbRounded];
                    
                    if (random3 == i) {
                        //Last run through of for block, replaces the placeholder ("@") with appendString. appendString doesn't get a placeholder ("@"), as another string will not be inserted
                        appendString = [NSString stringWithFormat: @"%li / %@", (long)random2, numb];
                        
                        //Finds range of placeholder.
                        NSRange range = [hourString rangeOfString:@"@"];
                        //Replaces @ with appendString.
                        [hourString replaceCharactersInRange: range withString: appendString];
                    } else if (i == 0) {
                        //First run through of for block, defines NSMutableString as what would be appendString, as there is nothing yet to be replaced.
                        hourString = [NSMutableString stringWithFormat: @"(@) / %@", numb];
                    } else {
                        //In between first and last run of for block, replaces the placeholder ("@") with appendString, and has appendString have a placeholder("@), for the next string to be inserted in.
                        appendString = [NSString stringWithFormat: @"(@) / %@", numb];
                        
                        //Finds range of placeholder.
                        NSRange range = [hourString rangeOfString:@"@"];
                        //Replaces @ with appendString.
                        [hourString replaceCharactersInRange: range withString: appendString];
                    }
                    
#                   ifdef GCDEBUGGER
                    NSLog (@"GCDEBUGGER MESSAGE: / i: %i answer: %ld _answer: %ld random2: %ld numb: %@ hourString: %@", i, (long)answer, (long)_answer, (long)random2, numb, hourString);
#                   endif
                    
                    _answer = random2;
                    
                    _hourNumber = hour;
                    stringHour = hourString;
                }
            }
            _stringForHour = hourString;
        } else
            stringHour = _stringForHour;
    }
    
    NSDateFormatter *dateFormatterMinute = [[NSDateFormatter alloc] init];
    [dateFormatterMinute setDateFormat:@"mm"];
    NSString *stringFromDateMinute = [dateFormatterMinute stringFromDate: date];
    unsigned int minute = [stringFromDateMinute intValue];

    
    NSString *stringMinute;
    if (prefNumber == 16)
        stringMinute = [NSString stringWithFormat:@"%X", minute];
    else if (prefNumber == 10)
        stringMinute = [NSString stringWithFormat:@"%u", minute];
    else if (prefNumber == 2){
        NSMutableString *str = [NSMutableString stringWithFormat:@""];
        for(NSInteger numberCopy = minute; numberCopy > 0; numberCopy >>=1) {
            //Prepend "0" or "1", depending on the bit
            [str insertString:((numberCopy & 1) ? @"1" : @"0") atIndex:0];
        }
        stringMinute = [NSString stringWithFormat:@"%@", str];
    } else if (prefNumber == 9) {
        //Convert to dec ASCII
        NSMutableString * str = [NSMutableString stringWithFormat:@""];
        NSString *minuteString = [NSString stringWithFormat:@"%i", minute];
        for (int i=0; i<[minuteString length]; i++) {
            char c = [minuteString characterAtIndex:i];
            int ASCII_Code = (int)c;
            NSString *ASCII_String = [NSString stringWithFormat: @"%i", ASCII_Code];
            [str appendString:ASCII_String];
            if (i + 1 != [minuteString length])
                [str appendString:@" "];
        }
#       ifdef GCDEBUGGER 
        NSLog (@"str minute: %@", str);
#       endif 
        stringMinute = str;
    } else if (prefNumber == 1) {
        //Convert to dec ASCII
        NSMutableString *str = [NSMutableString stringWithFormat:@""];
        NSString *minuteString = [NSString stringWithFormat:@"%i", minute];
        for (int i=0; i<[minuteString length]; ++i) {
            char c = [minuteString characterAtIndex:i];
            int ASCII_Code = (int)c;
            NSString *ASCII_String = [NSString stringWithFormat: @"%i", ASCII_Code];
            [str appendString:ASCII_String];
            if (i + 1 != [minuteString length])
                [str appendString:@" "];
        }
#       ifdef GCDEBUGGER
        NSLog (@"str minute: %@", str);
#       endif
        
        //Convert to binary
        NSMutableString *str2 = [NSMutableString stringWithFormat:@""];
        NSArray *array = [str componentsSeparatedByString: @" "];
        NSUInteger numObjects = [array count];
        //Converts each object of array into binary separately, so spaces remain intact.
        for (int i = 0; i < numObjects; ++i) {
            //Help for the translator from Martin R. http://stackoverflow.com/questions/19335367/converting-string-into-binary-with-spaces-intact
            NSString *string = array[i];
            NSMutableString *mutStr = [NSMutableString stringWithFormat:@""];
            for (NSInteger numberCopy = [string intValue]; numberCopy > 0; numberCopy >>=1) {
                //Prepend "0" or "1", depending on the bit
                [mutStr insertString:((numberCopy & 1) ? @"1" : @"0") atIndex:0];
            }
            [str2 appendString: mutStr];
            if (numObjects != i + 1) {
                //Add space if this is not last run through
                [str2 appendString: @" "];
            }
        }
        stringMinute = str2;
    } else if (prefNumber == 3) {
        //Minute Equation Mode is of medium difficulty.
        
        if (_minuteNumber != minute) {
            
            NSInteger random, random2;
            NSString *minuteString;
            
            //Decides which section to go through (addition, subtraction, etc)
            random = 1 + arc4random_uniform(4);
            
            
            if (random == 1) {
                //Addition
                if (minute == 0)
                    random2 = (NSInteger)(arc4random_uniform(61));
                else random2 = (NSInteger)(arc4random_uniform((minute + 1) * 100));
                
                CGFloat random2Float = random2;
                CGFloat float1 = random2Float / 1000;
                //Gets a NSNumber from float1, as NSNumber auto-truncates insignificant figures
                NSNumber *float1numb = [NSNumber numberWithFloat: float1];
                
                NSNumber *numb = [NSNumber numberWithFloat:(minute - float1)];
                minuteString = [NSString stringWithFormat: @" %@ + %@ ", numb, float1numb];
                
                _stringForMinute = minuteString;
                _minuteNumber = minute;
                stringMinute = minuteString;
            } else if (random == 2) {
                //Subtraction
                if (minute == 0)
                    random2 = (NSInteger)(arc4random_uniform(61));
                else random2 = (NSInteger)(arc4random_uniform(minute + 1) * 100);
                
                CGFloat random2Float = random2;
                CGFloat float1 = random2Float / 1000;
                //Gets a NSNumber from float1, as NSNumber auto-truncates insignificant figures
                NSNumber *float1numb = [NSNumber numberWithFloat: float1];
                
                NSNumber *numb = [NSNumber numberWithFloat:(minute + float1)];
                minuteString = [NSString stringWithFormat: @" %@ - %@ ", numb, float1numb];
                
                _stringForMinute = minuteString;
                _minuteNumber = minute;
                stringMinute = minuteString;
            } else if (random == 3) {
                //Multiplication
                if (minute == 0)
                    random2 = (NSInteger)(arc4random_uniform(15));
                else {
                    if ((minute + 1) <= 15)
                        random2 = 1 + (NSInteger)(arc4random_uniform(minute + 1));
                    else random2 = 1 + (NSInteger)(arc4random_uniform(15));
                }
                
                NSNumber *numb = [NSNumber numberWithFloat:(minute / random2)];
                minuteString = [NSString stringWithFormat: @" %@ * %ld ", numb, random2];
                
                _stringForMinute = minuteString;
                _minuteNumber = minute;
                stringMinute = minuteString;
            }  else if (random == 4) {
                //Division
                if (minute == 0)
                    random2 = (NSInteger)(arc4random_uniform(15));
                else {
                    if ((minute + 1) <= 15)
                        random2 = 1 + (NSInteger)(arc4random_uniform(minute + 1));
                    else random2 = 1 + (NSInteger)(arc4random_uniform(15));
                }
                
                NSNumber *numb = [NSNumber numberWithFloat:(minute * random2)];
                minuteString = [NSString stringWithFormat: @" %@ / %ld ", numb, random2];
                
                _stringForMinute = minuteString;
                _minuteNumber = minute;
                stringMinute = minuteString;
            }
        } else
            stringMinute = _stringForMinute;
    }
    
    NSDateFormatter * dateFormatterSecond = [[NSDateFormatter alloc] init];
    [dateFormatterSecond setDateFormat:@"ss"];
    NSString *stringFromDateSecond = [dateFormatterSecond stringFromDate: date];
    unsigned int second = [stringFromDateSecond intValue];
    
    
    NSString *stringSecond;
    if (prefNumber == 16)
        stringSecond = [NSString stringWithFormat:@"%X", second];
    else if (prefNumber == 10)
        stringSecond = [NSString stringWithFormat:@"%u", second];
    else if (prefNumber == 2){
        NSMutableString *str = [NSMutableString stringWithFormat:@""];
        for(NSInteger numberCopy = second; numberCopy > 0; numberCopy >>=1) {
            //Prepend "0" or "1", depending on the bit
            [str insertString:((numberCopy & 1) ? @"1" : @"0") atIndex:0];
        }
        stringSecond = [NSString stringWithFormat:@"%@", str];
    } else if (prefNumber == 9) {
        //Convert to dec ASCII 
        NSMutableString *str = [NSMutableString stringWithFormat:@""];
        NSString *secondString = [NSString stringWithFormat:@"%i", second];
        for (int i=0; i<[secondString length]; i++) {
            char c = [secondString characterAtIndex:i];
            int ASCII_Code = (int)c;
            NSString *ASCII_String = [NSString stringWithFormat: @"%i", ASCII_Code];
            [str appendString:ASCII_String];
            if (i + 1 != [secondString length])
                [str appendString:@" "];
        }
#       ifdef GCDEBUGGER
        NSLog (@"str second: %@", str);
#       endif 
        stringSecond = str;
    } else if (prefNumber == 1) {
        //Convert to dec ASCII
        NSMutableString * str = [NSMutableString stringWithFormat:@""];
        NSString *secondString = [NSString stringWithFormat:@"%i", second];
        for (int i=0; i<[secondString length]; i++) {
            char c = [secondString characterAtIndex:i];
            int ASCII_Code = (int)c;
            NSString *ASCII_String = [NSString stringWithFormat: @"%i", ASCII_Code];
            [str appendString:ASCII_String];
            if (i + 1 != [secondString length])
                [str appendString:@" "];
        }
#       ifdef GCDEBUGGER
        NSLog (@"str second: %@", str);
#       endif
        
        //Convert to binary
        NSMutableString *str2 = [NSMutableString stringWithFormat:@""];
        NSArray *array = [str componentsSeparatedByString: @" "];
        NSUInteger numObjects = [array count];
        //Converts each object of array into binary separately, so spaces remain intact.
        for (int i = 0; i  numObjects; ++i) {
            NSString *string = array[i];
            NSMutableString *mutStr = [NSMutableString stringWithFormat:@""];
            for (NSInteger numberCopy = [string intValue]; numberCopy > 0; numberCopy >>=1) {
                //Prepend "0" or "1", depending on the bit
                [mutStr insertString:((numberCopy & 1) ? @"1" : @"0") atIndex:0];
            }
            [str2 appendString: mutStr];
            if (numObjects != i + 1) {
                //Add space if this is not last run through
                [str2 appendString: @" "];
            }
        }
        stringSecond = str2;
    } else if (prefNumber == 3) {
        //Second Equation Mode is the easiest difficulty.
        NSInteger random, random2, numb;
        NSString *secondString;
            //Addition
            if (second == 0)
                random = (NSInteger)(arc4random_uniform(121)) - 60;
            else random = ((NSInteger)(arc4random_uniform(second * 2 + 1)) - 60);
            //Rounds random up to the least multiple of five that is greater or equal to random as multiples of five are easier to add.
            BOOL isMultipleOfFive = !(random % 5);
            while (isMultipleOfFive != YES) {
                ++random;
                isMultipleOfFive = !(random % 5);
            }
            numb = second - random;
        
            //Since if second is not a multiple of five, one of the numbers will be a multiple of five (random) and one will not (numb), random2 decides if random goes first in the string or if numb does
            random2 = 1 + arc4random_uniform(2);
            if (random2 == 1)
                secondString = [NSString stringWithFormat: @" %li + %li", numb, random];
            else if (random2 == 2)
                secondString = [NSString stringWithFormat: @" %li + %li", random, numb];
        stringSecond = secondString;
    }

    NSString *string;
    
    if ((prefNumber == 9) | (prefNumber == 1))
        string = [NSString stringWithFormat: @"(%@):(%@):(%@)", stringHour, stringMinute, stringSecond];
    else string = [NSString stringWithFormat: @"%@:%@:%@", stringHour, stringMinute, stringSecond];
    
    //Sets title to time
    [statusItem setTitle: string];
    
    [self performSelector: @selector(tick:) withObject: nil afterDelay: 1.0];
}

#pragma mark - MenuItems

-(IBAction)exit: (id)sender {
    //Quits Application. Used over [NSApp terminate: nil]; because this forces it to happen at the top of the next pass throught the event loop.
    [NSApp performSelector:@selector(terminate:) withObject:nil afterDelay: 0.0];
}

- (IBAction)loginItem:(id)sender {
    if (loginBool == YES) {
        loginBool = NO;
        [_loginItem setState: NSOffState];
        
        [GCLoginItemController removeAppFromLoginItemWithAllUsers: NO];
        
        NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
        
        NSString *number = [NSString stringWithFormat: @"%li", (long)preferenceNumber];
        BOOL loginBool = NO;
        NSString *string = [NSString stringWithFormat:@"%@@%hhd          ", number, loginBool];
        
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
        [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
        [fileHandle closeFile];
    } else {
        loginBool = YES;
        [_loginItem setState: NSOnState];
        
        [GCLoginItemController addAppAsLoginItemWithAllUsers:YES];
        
        NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
        
        NSString *number = [NSString stringWithFormat: @"%li", (long)preferenceNumber];
        BOOL loginBool = YES;
        NSString *string = [NSString stringWithFormat:@"%@@%hhd          ", number, loginBool];
        
        NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
        [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
        [fileHandle closeFile];
    }
}

-(IBAction)standardMode:(id)sender {
    preferenceNumber = 10;
    
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    
    NSString *number = @"10";
    NSString * string = [NSString stringWithFormat:@"%@@%hhd        ", number, loginBool];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
    [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
    [fileHandle closeFile];
    
    [_standardMenuItem setState: NSOnState];
    [_hexMenuItem setState: NSOffState];
    [_binaryMenuItem setState: NSOffState];
    [_decASCII_MenuItem setState: NSOffState];
    [_binASCII_MenuItem setState: NSOffState];
    [_equationMenuItem setState: NSOffState];
    
    [_standardMenuItem setEnabled: NO];
    [_hexMenuItem setEnabled: YES];
    [_binaryMenuItem setEnabled: YES];
    [_decASCII_MenuItem setEnabled: YES];
    [_binASCII_MenuItem setEnabled: YES];
    [_equationMenuItem setEnabled: YES];
}

-(IBAction)hexMode:(id)sender {
    preferenceNumber = 16;
    
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    
    NSString *number = @"16";
    NSString * string = [NSString stringWithFormat:@"%@@%hhd        ", number, loginBool];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
    [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
    [fileHandle closeFile];
    
    [_standardMenuItem setState: NSOffState];
    [_hexMenuItem setState: NSOnState];
    [_binaryMenuItem setState: NSOffState];
    [_decASCII_MenuItem setState: NSOffState];
    [_binASCII_MenuItem setState: NSOffState];
    [_equationMenuItem setState: NSOffState];
    
    [_standardMenuItem setEnabled: YES];
    [_hexMenuItem setEnabled: NO];
    [_binaryMenuItem setEnabled: YES];
    [_decASCII_MenuItem setEnabled: YES];
    [_binASCII_MenuItem setEnabled: YES];
    [_equationMenuItem setEnabled: YES];
}

-(IBAction)binaryMode:(id)sender {
    preferenceNumber = 2;
    
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    
    NSString *number = @"2";
    NSString * string = [NSString stringWithFormat:@"%@@%hhd        ", number, loginBool];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
    [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
    [fileHandle closeFile];
    
    [_standardMenuItem setState: NSOffState];
    [_hexMenuItem setState: NSOffState];
    [_binaryMenuItem setState: NSOnState];
    [_decASCII_MenuItem setState: NSOffState];
    [_binASCII_MenuItem setState: NSOffState];
    [_equationMenuItem setState: NSOffState];
    
    [_standardMenuItem setEnabled: YES];
    [_hexMenuItem setEnabled: YES];
    [_binaryMenuItem setEnabled: NO];
    [_decASCII_MenuItem setEnabled: YES];
    [_binASCII_MenuItem setEnabled: YES];
    [_equationMenuItem setEnabled: YES];
}

-(IBAction)decASCII_Mode:(id)sender {
    preferenceNumber = 9;
    
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    
    NSString *number = @"9";
    NSString * string = [NSString stringWithFormat:@"%@@%hhd        ", number, loginBool];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
    [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
    [fileHandle closeFile];
    
    [_standardMenuItem setState: NSOffState];
    [_hexMenuItem setState: NSOffState];
    [_binaryMenuItem setState: NSOffState];
    [_decASCII_MenuItem setState: NSOnState];
    [_binASCII_MenuItem setState: NSOffState];
    [_equationMenuItem setState: NSOffState];
    
    [_standardMenuItem setEnabled: YES];
    [_hexMenuItem setEnabled: YES];
    [_binaryMenuItem setEnabled: YES];
    [_decASCII_MenuItem setEnabled: NO];
    [_binASCII_MenuItem setEnabled: YES];
    [_equationMenuItem setEnabled: YES];
}

-(IBAction)binASCII_Mode:(id)sender {
    preferenceNumber  = 1;
    
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    
    NSString *number = @"1";
    NSString * string = [NSString stringWithFormat:@"%@@%hhd        ", number, loginBool];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
    [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
    [fileHandle closeFile];
    
    [_standardMenuItem setState: NSOffState];
    [_hexMenuItem setState: NSOffState];
    [_binaryMenuItem setState: NSOffState];
    [_decASCII_MenuItem setState: NSOffState];
    [_binASCII_MenuItem setState: NSOnState];
    [_equationMenuItem setState: NSOffState];
    
    [_standardMenuItem setEnabled: YES];
    [_hexMenuItem setEnabled: YES];
    [_binaryMenuItem setEnabled: YES];
    [_decASCII_MenuItem setEnabled: YES];
    [_binASCII_MenuItem setEnabled: NO];
    [_equationMenuItem setEnabled: YES];
}

-(IBAction)equationMode:(id)sender {
    preferenceNumber  = 3;
    
    NSString *fileName = [@"~/Library/Application Support/GClockData/Pref.txt" stringByExpandingTildeInPath];
    
    NSString *number = @"3";
    NSString * string = [NSString stringWithFormat:@"%@@%hhd        ", number, loginBool];
    NSFileHandle *fileHandle = [NSFileHandle fileHandleForWritingAtPath: fileName];
    [fileHandle writeData:[string dataUsingEncoding:NSUTF8StringEncoding]];
    [fileHandle closeFile];
    
    [_standardMenuItem setState: NSOffState];
    [_hexMenuItem setState: NSOffState];
    [_binaryMenuItem setState: NSOffState];
    [_decASCII_MenuItem setState: NSOffState];
    [_binASCII_MenuItem setState: NSOffState];
    [_equationMenuItem setState: NSOnState];
    
    [_standardMenuItem setEnabled: YES];
    [_hexMenuItem setEnabled: YES];
    [_binaryMenuItem setEnabled: YES];
    [_decASCII_MenuItem setEnabled: YES];
    [_binASCII_MenuItem setEnabled: YES];
    [_equationMenuItem setEnabled: NO];
}

        
@end

